'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const core = require('@envelop/core');
const object = require('@graphql-ez/utils/object');
const smartCache = require('./smart-cache.js');
const utils = require('@graphql-ez/utils');
const types = require('@envelop/types');

const InternalAppBuildContextKey = Symbol("graphql-ez-internal-app-build-context");
function createEZAppFactory(factoryCtx, rawOptionsArg, {
  preBuild,
  afterBuild
} = {}) {
  var _a, _b, _c, _d, _e;
  const presets = object.toPlural((_a = rawOptionsArg.ez) == null ? void 0 : _a.preset);
  const rawOptions = {
    ...Object.assign(rawOptionsArg, ...presets.map((v) => v.options && object.cleanObject(v.options)), { ...rawOptionsArg })
  };
  const envelopPluginsPre = [...object.toPlural((_b = rawOptions.envelop) == null ? void 0 : _b.plugins)];
  const ezPluginsDirty = [...object.toPlural((_c = rawOptions.ez) == null ? void 0 : _c.plugins)];
  const envelopPresets = object.toPlural((_d = rawOptions.envelop) == null ? void 0 : _d.preset);
  for (const envelopPreset of envelopPresets) {
    envelopPluginsPre.unshift(core.useEnvelop(envelopPreset));
  }
  for (const preset of presets) {
    if (preset.self)
      ezPluginsDirty.push(preset.self);
    if (preset.ezPlugins)
      ezPluginsDirty.push(...preset.ezPlugins);
    if (preset.envelopPlugins)
      envelopPluginsPre.push(...preset.envelopPlugins);
  }
  const integrationName = factoryCtx.integrationName;
  const ezPluginsPre = ezPluginsDirty.filter((pluginValue, index) => {
    if (typeof pluginValue !== "object" || pluginValue == null)
      return false;
    const { name, compatibilityList } = pluginValue;
    if (compatibilityList) {
      const isCompatible = compatibilityList[integrationName];
      if (isCompatible instanceof Error)
        throw isCompatible;
      else if (!isCompatible)
        throw Error(`[graphql-ez] "${name}" is not compatible with "${integrationName}"`);
    }
    if (ezPluginsDirty.findIndex((plugin) => typeof plugin === "object" && plugin != null && plugin.name === name) === index)
      return true;
    console.warn(`[graphql-ez] Warning! Plugin "${name}" is duplicated! Make sure to specify the same plugin only once.`);
    return false;
  });
  const envelopPlugins = [...envelopPluginsPre];
  const ezPlugins = Object.freeze([...ezPluginsPre]);
  const baseAppBuilder = {
    gql: utils.gql,
    registerDataLoader() {
      throw Error(`[graphql-ez] To use "registerDataLoader" you have to add the "ezDataLoader" plugin first!`);
    },
    registerModule() {
      throw Error(`[graphql-ez] To use "registerModule" you have to add the "ezGraphQLModules" plugin first!`);
    },
    registerTypeDefs() {
      throw Error(`[graphql-ez] To use "registerTypeDefs" you have to add the "ezSchema" plugin first!`);
    },
    registerResolvers() {
      throw Error(`[graphql-ez] To use "registerResolvers" you have to add the "ezSchema" plugin first!`);
    },
    registerSchema() {
      throw Error(`[graphql-ez] To use "registerSchema" you have to add the "ezSchema" plugin first!`);
    },
    get asPreset() {
      return {
        options: object.cleanObject({ ...rawOptions, ez: void 0, envelop: void 0 }),
        ezPlugins: [...ezPluginsPre],
        envelopPlugins: [...envelopPluginsPre]
      };
    },
    envelopPlugins,
    ezPlugins,
    modulesApplication: utils.LazyPromise(() => {
      throw Error('[graphql-ez] To use "modulesApplication" you have to add the "ezGraphQLModules" plugin first!');
    }),
    [InternalAppBuildContextKey]: null
  };
  const options = {
    ...rawOptions,
    ez: {
      plugins: ezPlugins
    },
    envelop: {
      plugins: envelopPlugins,
      presets: envelopPresets,
      enableInternalTracing: (_e = rawOptions.envelop) == null ? void 0 : _e.enableInternalTracing
    }
  };
  const ctx = {
    ...factoryCtx,
    options,
    appBuilder: baseAppBuilder
  };
  baseAppBuilder[InternalAppBuildContextKey] = ctx;
  let registerError;
  const registerPromise = Promise.all(ezPlugins.map((plugin) => {
    var _a2;
    return (_a2 = plugin.onRegister) == null ? void 0 : _a2.call(plugin, ctx);
  })).catch((err) => {
    registerError = err;
  });
  let isBuilt = false;
  const appBuilder = async function appBuilder2(buildOptions = {}, adapterFactory) {
    if (isBuilt)
      throw Error(`[graphql-ez] You can't call "buildApp" more than once for a single EZ App instance.`);
    isBuilt = true;
    await registerPromise;
    if (registerError)
      throw registerError;
    if (buildOptions.prepare)
      await buildOptions.prepare(baseAppBuilder);
    if (options.prepare)
      await options.prepare(baseAppBuilder);
    await Promise.all([
      ...ezPlugins.map((plugin) => {
        var _a2;
        return (_a2 = plugin.onPreBuild) == null ? void 0 : _a2.call(plugin, ctx);
      }),
      preBuild == null ? void 0 : preBuild(ctx),
      smartCache.SmartCacheIntrospection()(ctx)
    ]);
    if (!ctx.schemaPlugin && options.schema && options.schema !== "dynamic") {
      envelopPlugins.push(core.useSchema(await options.schema));
    }
    Object.freeze(envelopPlugins);
    const getEnveloped = core.envelop({
      plugins: (await Promise.all(envelopPlugins)).filter((plugin) => {
        return plugin != null && typeof plugin != "boolean";
      }),
      enableInternalTracing: ctx.options.envelop.enableInternalTracing
    });
    if (options.schema !== "dynamic" && !getEnveloped().schema) {
      throw Error(`[graphql-ez] No GraphQL Schema specified!. If you are using a dynamic schema, make sure to set the "schema" configuration property as "dynamic".`);
    }
    await Promise.all([
      ...ezPlugins.map((plugin) => {
        var _a2;
        return (_a2 = plugin.onAfterBuild) == null ? void 0 : _a2.call(plugin, getEnveloped, ctx);
      }),
      afterBuild == null ? void 0 : afterBuild(getEnveloped, ctx)
    ]);
    return {
      app: adapterFactory({
        getEnveloped,
        ctx
      }),
      getEnveloped
    };
  };
  async function onIntegrationRegister(integrationCtx) {
    const integration = integrationCtx[integrationName];
    if (integration == null)
      throw Error(`Error on @graphql-ez/${integrationName}!`);
    await Promise.all(ezPlugins.map(async ({ onIntegrationRegister: onIntegrationRegister2 }) => {
      var _a2;
      if (!onIntegrationRegister2)
        return;
      const integrationCallback = (_a2 = await onIntegrationRegister2(ctx)) == null ? void 0 : _a2[integrationName];
      return integrationCallback == null ? void 0 : integrationCallback({
        ctx,
        integration
      });
    }));
  }
  return { ...baseAppBuilder, appBuilder, onIntegrationRegister };
}

exports.InternalAppBuildContextKey = InternalAppBuildContextKey;
exports.createEZAppFactory = createEZAppFactory;
for (const k in types) {
  if (k !== 'default' && !exports.hasOwnProperty(k)) Object.defineProperty(exports, k, {
    enumerable: true,
    get: function () { return types[k]; }
  });
}
