import { getGraphQLParameters, processRequest } from '@pablosz/graphql-helix';

async function handleRequest({
  request,
  getEnveloped,
  contextArgs,
  buildContext,
  onResponse,
  onMultiPartResponse,
  onPushResponse,
  baseOptions,
  processRequestOptions,
  preProcessRequest
}) {
  const contextArgsData = contextArgs();
  const { parse, validate, contextFactory, execute, schema, subscribe } = getEnveloped(buildContext ? Object.assign(contextArgsData, await buildContext(contextArgsData)) : contextArgsData);
  if (Array.isArray(request.body)) {
    const allowBatchedQueries = baseOptions.allowBatchedQueries;
    if (!allowBatchedQueries)
      throw Error("Batched queries not enabled!");
    if (typeof allowBatchedQueries === "number" && request.body.length > allowBatchedQueries) {
      throw Error("Batched queries limit exceeded!");
    }
    const HeadersMap = {};
    const payload = await Promise.all(request.body.map(async (body) => {
      const { operationName: operationName2, query: query2, variables: variables2, extensions: extensions2 } = getGraphQLParameters({ ...request, body });
      const options2 = {
        operationName: operationName2,
        query: query2,
        variables: variables2,
        request,
        schema,
        parse,
        validate,
        contextFactory,
        execute,
        subscribe,
        extensions: extensions2
      };
      if (processRequestOptions) {
        Object.assign(options2, processRequestOptions());
      }
      let result2;
      if (preProcessRequest) {
        for (const handler of preProcessRequest) {
          const handlerResult = await handler({
            appOptions: baseOptions,
            requestOptions: options2
          });
          if (handlerResult) {
            result2 = handlerResult;
            break;
          }
        }
        if (!result2)
          result2 = await processRequest(options2);
      } else {
        result2 = await processRequest(options2);
      }
      if (result2.type !== "RESPONSE")
        throw Error(`Unsupported ${result2.type} in Batched Queries!`);
      for (const { name, value } of result2.headers) {
        HeadersMap[name] = value;
      }
      return result2.payload;
    }));
    return onResponse({
      type: "RESPONSE",
      status: 200,
      payload,
      headers: Object.entries(HeadersMap).map(([name, value]) => ({ name, value }))
    }, defaultResponseHandle);
  }
  const { operationName, query, variables, extensions } = getGraphQLParameters(request);
  const options = {
    operationName,
    query,
    variables,
    request,
    schema,
    parse,
    validate,
    contextFactory,
    execute,
    subscribe,
    extensions
  };
  if (processRequestOptions) {
    Object.assign(options, processRequestOptions());
  }
  let result;
  if (preProcessRequest) {
    for (const handler of preProcessRequest) {
      const handlerResult = await handler({
        appOptions: baseOptions,
        requestOptions: options
      });
      if (handlerResult) {
        result = handlerResult;
        break;
      }
    }
    if (!result)
      result = await processRequest(options);
  } else {
    result = await processRequest(options);
  }
  switch (result.type) {
    case "RESPONSE": {
      return onResponse(result, defaultResponseHandle);
    }
    case "MULTIPART_RESPONSE": {
      return onMultiPartResponse(result, defaultMultipartResponseHandle);
    }
    case "PUSH": {
      return onPushResponse(result, defaultPushResponseHandle);
    }
  }
}
function defaultResponseHandle(_req, res, result) {
  res.writeHead(result.status, {
    "content-type": "application/json"
  });
  for (const { name, value } of result.headers) {
    res.setHeader(name, value);
  }
  res.end(JSON.stringify(result.payload));
}
async function defaultMultipartResponseHandle(req, res, result) {
  res.writeHead(200, {
    Connection: "keep-alive",
    "Content-Type": 'multipart/mixed; boundary="-"',
    "Transfer-Encoding": "chunked",
    "Content-Encoding": "none"
  });
  req.socket.on("close", () => {
    result.unsubscribe();
  });
  res.write("---");
  await result.subscribe((result2) => {
    const chunk = Buffer.from(JSON.stringify(result2), "utf8");
    const data = ["", "Content-Type: application/json; charset=utf-8", "Content-Length: " + String(chunk.length), "", chunk];
    if (result2.hasNext) {
      data.push("---");
    }
    res.write(data.join("\r\n"));
  });
  res.write("\r\n-----\r\n");
  res.end();
}
async function defaultPushResponseHandle(req, res, result) {
  res.writeHead(200, {
    "Content-Encoding": "none",
    "Content-Type": "text/event-stream",
    Connection: "keep-alive",
    "Cache-Control": "no-cache"
  });
  req.socket.on("close", () => {
    result.unsubscribe();
  });
  await result.subscribe((result2) => {
    res.write(`data: ${JSON.stringify(result2)}

`);
  });
}

export { defaultMultipartResponseHandle, defaultPushResponseHandle, defaultResponseHandle, handleRequest };
