import { getObjectValue } from '@graphql-ez/utils/object';
import { NoSchemaIntrospectionCustomRule, print } from 'graphql';
import lru from 'tiny-lru';

const DEFAULT_MAX = 1e3;
const DEFAULT_TTL = 36e5;
const SmartCacheIntrospection = () => {
  const DocumentKeyMap = /* @__PURE__ */ new WeakMap();
  let isValidationCacheDisabled;
  const useParserCache = (pluginOptions = {}) => {
    const max = typeof pluginOptions.max === "number" ? pluginOptions.max : DEFAULT_MAX;
    const ttl = typeof pluginOptions.ttl === "number" ? pluginOptions.ttl : DEFAULT_TTL;
    const documentCache = lru(max, ttl);
    const errorCache = lru(max, ttl);
    return {
      onParse({ params: { source }, setParsedDocument }) {
        const key = typeof source === "object" ? source.body : source;
        const errorValue = errorCache.get(key);
        if (errorValue)
          throw errorValue;
        const documentCacheValue = documentCache.get(key);
        if (documentCacheValue)
          setParsedDocument(documentCacheValue);
        return ({ result }) => {
          if (result instanceof Error) {
            errorCache.set(key, result);
          } else if (result !== null) {
            documentCache.set(key, result);
            DocumentKeyMap.set(result, key);
          }
        };
      }
    };
  };
  const useValidationCache = (pluginOptions = {}) => {
    const max = typeof pluginOptions.max === "number" ? pluginOptions.max : DEFAULT_MAX;
    const ttl = typeof pluginOptions.ttl === "number" ? pluginOptions.ttl : DEFAULT_TTL;
    const resultCache = lru(max, ttl);
    return {
      onSchemaChange() {
        resultCache.clear();
      },
      onValidate({ params: { documentAST }, setResult }) {
        let key = DocumentKeyMap.get(documentAST);
        if (key == null)
          key = print(documentAST);
        const cacheValue = resultCache.get(key);
        if (cacheValue)
          setResult(cacheValue);
        return ({ result }) => {
          resultCache.set(key, result);
        };
      }
    };
  };
  const useDisableIntrospection = (options) => {
    const disableIf = options == null ? void 0 : options.disableIf;
    return {
      onValidate: disableIf ? ({ addValidationRule, context, params }) => {
        const shouldDisableIntrospection = disableIf({ context, params });
        if (shouldDisableIntrospection)
          addValidationRule(NoSchemaIntrospectionCustomRule);
        if (isValidationCacheDisabled)
          return;
        const { documentAST } = params;
        const disableIntrospectionFlag = shouldDisableIntrospection ? "1" : "0";
        const documentKey = DocumentKeyMap.get(documentAST);
        if (documentKey) {
          DocumentKeyMap.set(documentAST, documentKey + disableIntrospectionFlag);
        } else {
          DocumentKeyMap.set(documentAST, print(documentAST) + disableIntrospectionFlag);
        }
      } : ({ addValidationRule }) => {
        addValidationRule(NoSchemaIntrospectionCustomRule);
      }
    };
  };
  return function smartCache(ctx) {
    const cache = ctx.options.cache;
    const envelopPlugins = ctx.options.envelop.plugins;
    const introspection = ctx.options.introspection;
    const isParserEnabled = cache ? cache === true || !!cache.parse : false;
    const isValidationEnabled = cache ? cache === true || !!cache.validation : false;
    isValidationCacheDisabled = !isValidationEnabled;
    if (introspection == null ? void 0 : introspection.disable) {
      envelopPlugins.push(useDisableIntrospection(getObjectValue(introspection.disable)));
    }
    if (cache) {
      const cacheObj = getObjectValue(cache);
      if (isParserEnabled) {
        envelopPlugins.push(useParserCache(getObjectValue(cacheObj == null ? void 0 : cacheObj.parse)));
      }
      if (isValidationEnabled) {
        envelopPlugins.push(useValidationCache(getObjectValue(cacheObj == null ? void 0 : cacheObj.validation)));
      }
    }
  };
};

export { SmartCacheIntrospection };
